/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.binding.soap.listener.incoming.servlet;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.transform.TransformerException;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.transport.http.AxisServlet;
import org.ow2.petals.binding.soap.listener.incoming.PetalsAxisService;
import org.ow2.petals.binding.soap.listener.incoming.jetty.ServerStats;
import org.ow2.petals.commons.threadlocal.DocumentBuilders;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.util.WSDLHelper;
import org.ow2.petals.component.framework.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Special servlet for Jetty server. This servlet extends AxisServlet to
 * override the initConfigContext method that load a configuration context from
 * a War file. In the SoapComponent, the configuration context is loaded at
 * component startup and is passed by Jetty server and this servlet.
 * 
 * @author Christophe Hamerling - EBM WebSourcing
 * 
 */
public class SoapServlet extends AxisServlet {

    // TODO: configContext is reset during Servlet.init by AxisServlet, so an
    // other
    // variable must be used to save the configuration.
    protected transient ConfigurationContext copyOfconfigContext;

    private final ServerStats stats;

    /**
     * 
     */
    private static final long serialVersionUID = -8035921158344750120L;

    /**
     * Creates a new instance of SoapServlet
     * 
     * @param configContext
     * @param stats
     */
    public SoapServlet(final ConfigurationContext configContext, final ServerStats stats) {
        this.copyOfconfigContext = configContext;
        this.stats = stats;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.http.AxisServlet#initConfigContext(javax.servlet.ServletConfig)
     */
    @Override
    protected ConfigurationContext initConfigContext(final ServletConfig config)
            throws ServletException {
        // nothing to do, just return the configuration context. Avoid loading
        // the configuration from default location (WAR)
        this.copyOfconfigContext.setProperty(Constants.CONTAINER_MANAGED, Constants.VALUE_TRUE);
        return this.copyOfconfigContext;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.http.AxisServlet#doGet(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    @Override
    protected void doGet(final HttpServletRequest request, final HttpServletResponse response)
            throws ServletException, IOException {
        String queryString = request.getQueryString();
        if (isUpperWSDLRequest(queryString)) {
            response.sendRedirect(buildRedirect(request));
            return;
        } else if (isImportWSDLRequest(queryString)) {
            this.printImportedDocuments(request, response, queryString);
        } else {
            this.stats.newGetRequest();
            super.doGet(request, response);
        }
    }

    /**
     * Get and prints the imported document from WSDL
     * 
     * @param request
     * @param response
     * @param queryString
     * @throws AxisFault
     * @throws IOException
     * @throws ServletException
     */
    private void printImportedDocuments(final HttpServletRequest request,
            final HttpServletResponse response, String queryString) throws AxisFault, IOException,
            ServletException {
        AxisConfiguration axisConfig = this.axisConfiguration;
        // try to find if the axisService has already been registered
        AxisService axisService = axisConfig.getService(createServiceName(request));
        if (axisService != null) {
            if (axisService instanceof PetalsAxisService) {
                List<Element> elmts = ((PetalsAxisService) axisService).getImportedDocuments();
                boolean find = false;
                Iterator<Element> it = elmts.iterator();
                Element elmt = null;
                String docFileName = queryString.replace("wsdl=", "");
                while ((!find) && (it.hasNext())) {
                    elmt = it.next();

                    if (elmt.getAttributeNode("name") != null) {
                        if (elmt.getAttributeNode("name").getNodeValue().equals(docFileName)) {
                            find = true;
                            NodeList list = elmt.getChildNodes();
                            if (list.getLength() > 0) {
                                if (list.item(0) instanceof Element) {
                                    elmt = (Element) list.item(0);
                                    try {
                                        Document doc = DocumentBuilders.getDefaultDocumentBuilder()
                                                .newDocument();
                                        doc.adoptNode(elmt);
                                        doc.appendChild(elmt);
                                        doc = WSDLHelper.changeImportLocation(doc,
                                                ((PetalsAxisService) axisService).getLocalAddress()
                                                        + "/" + axisService.getName() + "?wsdl=");

                                        String res = XMLUtil.createStringFromDOMDocument(doc);
                                        final ServletOutputStream out = response.getOutputStream();
                                        out.write(res.getBytes());
                                        out.flush();
                                        out.close();
                                    } catch (TransformerException e) {
                                        throw new ServletException("Error to print document: "
                                                + docFileName, e);
                                    } catch (PEtALSCDKException e) {
                                        throw new ServletException("Error to print document: "
                                                + docFileName, e);
                                    }
                                }
                            }
                        }
                    }
                }
                if (!find) {
                    throw new ServletException("Error: Document unknown: " + docFileName);
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.http.AxisServlet#doPost(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    @Override
    protected void doPost(final HttpServletRequest request, final HttpServletResponse response)
            throws ServletException, IOException {
        this.stats.newPostRequest();
        super.doPost(request, response);
    }

    /**
     * 
     * @param request
     * @return
     */
    private boolean isUpperWSDLRequest(String queryString) {
        return (queryString != null && (queryString.indexOf("WSDL") >= 0 || queryString
                .indexOf("WSDL2") >= 0));
    }
    
    /**
     * 
     * @param queryString
     * @return
     */
    private boolean isImportWSDLRequest(String queryString) {
        return (queryString != null) && (queryString.toLowerCase().startsWith("wsdl="));
    }

    /**
     * 
     * @param request
     * @return
     */
    private String buildRedirect(HttpServletRequest request) {
        String result = "http://" + request.getServerName() + ":" + request.getServerPort();
        String query = request.getQueryString();
        int index = query.lastIndexOf("WSDL");
        if (index >= 0) {
            query = query.replaceAll("WSDL", "wsdl");
        }
        return result + request.getRequestURI() + "?" + query;
    }

    /**
     * 
     * @param request
     * @return
     */
    private String createServiceName(HttpServletRequest request) {
        String remove = request.getContextPath() + request.getServletPath() + "/";
        String res = request.getRequestURI().replace(remove, "");
        return res;
    }
}
